/*
Copyright © 2011-2012 Clint Bellanger

This file is part of FLARE.

FLARE is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

FLARE is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
FLARE.  If not, see http://www.gnu.org/licenses/
*/

#include <iostream>

#include "GameStateLoad.h"
#include "GameStateTitle.h"
#include "GameStateConfig.h"
#include "SharedResources.h"
#include "Settings.h"
#include "WidgetButton.h"
#include "WidgetLabel.h"
#include "WidgetTooltip.h"

GameStateTitle::GameStateTitle() : GameState() {

	exit_game = false;
	load_game = false;

	loadGraphics();

	// set up buttons
	button_play = new WidgetButton(mods->locate("images/menus/buttons/button_default.png"));
	button_exit = new WidgetButton(mods->locate("images/menus/buttons/button_default.png"));
	button_cfg = new WidgetButton(mods->locate("images/menus/buttons/button_default.png"));

	button_play->label = msg->get("Play Game");
	button_play->pos.x = VIEW_W_HALF - button_play->pos.w/2;
	button_play->pos.y = VIEW_H - (button_exit->pos.h*3);
	if (!ENABLE_PLAYGAME) button_play->enabled = false;
	button_play->refresh();

	button_cfg->label = msg->get("Configuration");
	button_cfg->pos.x = VIEW_W_HALF - button_cfg->pos.w/2;
	button_cfg->pos.y = VIEW_H - (button_exit->pos.h*2);
	button_cfg->refresh();

	button_exit->label = msg->get("Exit Game");
	button_exit->pos.x = VIEW_W_HALF - button_exit->pos.w/2;
	button_exit->pos.y = VIEW_H - button_exit->pos.h;
	button_exit->refresh();

	// set up labels
	label_version = new WidgetLabel();
	label_version->set(VIEW_W, 0, JUSTIFY_RIGHT, VALIGN_TOP, msg->get("Flare Alpha v0.17"), font->getColor("menu_normal"));

	// set up tooltip
	tip = new WidgetTooltip();
	tipdata = new TooltipData();
	tipdata->num_lines = 1;
}

void GameStateTitle::loadGraphics() {

	SDL_Surface *cleanup = IMG_Load(mods->locate("images/menus/logo.png").c_str());

	if(!cleanup) {
		fprintf(stderr, "Couldn't load image: %s\n", IMG_GetError());
		SDL_Quit();
		exit(1);
	}

	// optimize
	logo = SDL_DisplayFormatAlpha(cleanup);
	SDL_FreeSurface(cleanup);
}

void GameStateTitle::logic() {

	if (ENABLE_PLAYGAME) {
		button_play->enabled = true;
		if (inpt->pressing[ACCEPT] && !inpt->lock[ACCEPT]) {
			inpt->lock[ACCEPT] = true;
			delete requestedGameState;
			requestedGameState = new GameStateLoad();
		} else if (inpt->pressing[CANCEL] && !inpt->lock[CANCEL]) {
			inpt->lock[CANCEL] = true;
			exitRequested = true;
		}
	}

	
	if (button_play->checkClick()) {
		delete requestedGameState;
		requestedGameState = new GameStateLoad();
	} else if (button_cfg->checkClick()) {
		delete requestedGameState;
		requestedGameState = new GameStateConfig();
	} else if (button_exit->checkClick()) {
		exitRequested = true;
	}
	
	// handle tooltip
	if (!ENABLE_PLAYGAME && button_play->hover) {
		tipdata->lines[0] = msg->get("Enable a core mod to continue");
	}
	else {
		tipdata->lines[0] = "";
	}
}

void GameStateTitle::render() {

	SDL_Rect src;
	SDL_Rect dest;

	// display logo centered
	src.x = src.y = 0;
	src.w = dest.w = logo->w;
	src.h = dest.h = logo->h;
	dest.x = VIEW_W_HALF - (logo->w/2);
	dest.y = VIEW_H_HALF - (logo->h/2);
	SDL_BlitSurface(logo, &src, screen, &dest);

	// display buttons
	button_play->render();
	button_cfg->render();
	button_exit->render();

	// version number
	label_version->render();
	
	// display tooltip
	if (tipdata->lines[0] != "") {
		tip->render(*tipdata, inpt->mouse, STYLE_FLOAT, screen);
	}
	
}

GameStateTitle::~GameStateTitle() {
	delete button_play;
	delete button_cfg;
	delete button_exit;
	delete label_version;
	SDL_FreeSurface(logo);
	delete tip;
	delete tipdata;
}
